"""
ice_pick.filters.usage_types
~~~~~~~~~~~~~~~~~~~~~~~~~~~~

This module contains a set of constants that can be used as usage type
filters. All the constants are based on AWS definitions.

Copyright 2014 Demand Media.

    Licensed under the Apache License, Version 2.0 (the "License");
    you may not use this file except in compliance with the License.
    You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

    Unless required by applicable law or agreed to in writing, software
    distributed under the License is distributed on an "AS IS" BASIS,
    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
    See the License for the specific language governing permissions and
    limitations under the License.
"""


AP_DATA_TRANSFER_OUT_BYTES = 'AP-DataTransfer-Out-Bytes'
AP_REQUEST_TIER_1 = 'AP-Requests-Tier1'
AP_REQUEST_TIER_2_HTTPS = 'AP-Requests-Tier2-HTTPS'
APN1_AWS_IN_BYTES = 'APN1-AWS-In-Bytes'
APN1_AWS_OUT_BYTES = 'APN1-AWS-Out-Bytes'
APS1_AWS_IN_BYTES = 'APS1-AWS-In-Bytes'
APS1_AWS_OUT_BYTES = 'APS1-AWS-Out-Bytes'
APS2_AWS_IN_BYTES = 'APS2-AWS-In-Bytes'
APS2_AWS_OUT_BYTES = 'APS2-AWS-Out-Bytes'
AU_DATA_TRANSFER_OUT_BYTES = 'AU-DataTransfer-Out-Bytes'
AU_REQUESTS_TIER1 = 'AU-Requests-Tier1'
AU_REQUESTS_TIER2_HTTPS = 'AU-Requests-Tier2-HTTPS'
AWS_ACTIVITIES_FREQ = 'AWS-Activities-freq'
AWS_ACTIVITIES_INFREQ = 'AWS-Activities-infreq'
ATTACHMENTS_SIZE_BYTES = 'AttachmentsSize-Bytes'
BOX_USAGE = 'BoxUsage'
C1_MEDIUM = 'c1.medium'
C1_XLARGE = 'c1.xlarge'
C3_4XLARGE_WINDOWS = 'c3.4xlarge.windows'
C3_8XLARGE_WINDOWS = 'c3.8xlarge.windows'
C3_XLARGE = 'c3.xlarge'
C3_XLARGE_WINDOWS = 'c3.xlarge.windows'
CW_ALARM_MONITOR_USAGE = 'CW:AlarmMonitorUsage'
CW_METRIC_MONITOR_USAGE = 'CW:MetricMonitorUsage'
CW_REQUESTS = 'CW:Requests'
CLOSED_RETAINED_WORKFLOW_DAYS = 'ClosedRetainedWorkflowDays'
CLOUD_FRONT_IN_BYTES = 'CloudFront-In-Bytes'
CLOUD_FRONT_OUT_BYTES = 'CloudFront-Out-Bytes'
DNS_QUERIES = 'DNS-Queries'
DATA_PROCESSING_BYTES = 'DataProcessing-Bytes'
DATA_TRANSFER_IN_BYTES = 'DataTransfer-In-Bytes'
DATA_TRANSFER_OUT_BYTES = 'DataTransfer-Out-Bytes'
DATA_TRANSFER_REGIONAL_BYTES = 'DataTransfer-Regional-Bytes'
DELIVERY_ATTEMPS_SMTP = 'DeliveryAttempts-SMTP'
DW_HS1_XLARGE = 'dw.hs1.xlarge'
EBS_SNAPSHOT_USAGE = 'EBS:SnapshotUsage'
EBS_VOLUMEN_IO_USAGE = 'EBS:VolumeIOUsage'
EBS_VOLUMEN_P_IOPS_PIOPS = 'EBS:VolumeP-IOPS.piops'
EBS_VOLUMEN_USAGE = 'EBS:VolumeUsage'
EBS_VOLUMEN_USAGE_PIOPS = 'EBS:VolumeUsage.piops'
EBS_OPTIMIZED_M1_LARGE = 'EBSOptimized:m1.large'
EU_AWS_IN_BYTES = 'EU-AWS-In-Bytes'
EU_AWS_OUT_BYTES = 'EU-AWS-Out-Bytes'
EARLY_DELETE_BYTE_HRS = 'EarlyDelete-ByteHrs'
ELASTIC_IP_IDLE_ADDRESS = 'ElasticIP:IdleAddress'
ELASTIC_IP_REMAP = 'ElasticIP:Remap'
HOSTED_ZONE = 'HostedZone'
IN_DATA_TRANSFER_OUT_BYTES = 'IN-DataTransfer-Out-Bytes'
IN_REQUEST_TIER1 = 'IN-Requests-Tier1'
IN_REQUEST_TIER2_HTTPS = 'IN-Requests-Tier2-HTTPS'
INACTIVE_PIPELINES = 'InactivePipelines'
INITIATED_ACTIONS = 'InitiatedActions'
INITIATED_WORKFLOW = 'InitiatedWorkflows'
INSTANCE_USAGE_DB_M1_LARGE = 'InstanceUsage:db.m1.large'
INSTANCE_USAGE_DB_M1_SMALL = 'InstanceUsage:db.m1.small'
INSTANCE_USAGE_DB_M1_XLARGE = 'InstanceUsage:db.m1.xlarge'
INSTANCE_USAGE_DB_M2_2XLARGE = 'InstanceUsage:db.m2.2xlarge'
INSTANCE_USAGE_DB_T1_MICRO = 'InstanceUsage:db.t1.micro'
INTRA_AWS_DNS_QUERIES = 'Intra-AWS-DNS-Queries'
JP_DATA_TRANSFER_OUT_BYTES = 'JP-DataTransfer-Out-Bytes'
JP_REQUESTS_TIER1 = 'JP-Requests-Tier1'
JP_REQUESTS_TIER2_HTTPS = 'JP-Requests-Tier2-HTTPS'
LOAD_BALANCER_USAGE = 'LoadBalancerUsage'
M1_LARGE = 'm1.large'
M1_LARGE_WINDOWS = 'm1.large.windows'
M1_MEDIUM = 'm1.medium'
M1_SMALL = 'm1.small'
M1_SMALL_WINDOWS = 'm1.small.windows'
M1_XLARGE = 'm1.xlarge'
M2_2XLARGE = 'm2.2xlarge'
M2_4XLARGE = 'm2.4xlarge'
M2_4X_LARGE_WINDOWS = 'm2.4xlarge.windows'
M2_XLARGE = 'm2.xlarge'
M3_2XLARGE = 'm3.2xlarge'
M3_XLARGE = 'm3.xlarge'
MULTI_AZ_USAGE_DB_M1_LARGE = 'Multi-AZUsage:db.m1.large'
MULTI_AZ_USAGE_DB_M1_SMALL = 'Multi-AZUsage:db.m1.small'
MULTI_AZ_USAGE_DB_M1_XLARGE = 'Multi-AZUsage:db.m1.xlarge'
MULTI_AZ_USAGE_DB_M2_2XLARGE = 'Multi-AZUsage:db.m2.2xlarge'
NODE_USAGE_CACHE_M1_SMALL = 'NodeUsage:cache.m1.small'
NODE_USAGE_CACHE_M3_2XLARGE = 'NodeUsage:cache.m3.2xlarge'
OPEN_WORKFLOW_DAYS = 'OpenWorkflowDays'
PUT_REQUEST_VOLUMEN = 'PutRequestVolume'
RDS_MULTI_AZ_PIOPS = 'RDS:Multi-AZ-PIOPS'
RDS_MULTI_AZ_PIOPS_STORAGE = 'RDS:Multi-AZ-PIOPS-Storage'
RDS_MULTI_AZ_STORAGE_USAGE = 'RDS:Multi-AZ-StorageUsage'
RDS_STORAGE_IOUSAGE = 'RDS:StorageIOUsage'
RDS_STORAGE_USAGE = 'RDS:StorageUsage'
READ_CAPACITY_UNIT_HRS = 'ReadCapacityUnit-Hrs'
RECIPIENTS = 'Recipients'
RECIPIENTS_EC2 = 'Recipients-EC2'
REQUEST = 'request'
REQUEST_RBP = 'Requests-RBP'
REQUESTS_TIER1 = 'Requests-Tier1'
REQUESTS_TIER2 = 'Requests-Tier2'
REQUESTS_TIER2_HTTPS = 'Requests-Tier2-HTTPS'
REQUEST_TIER3 = 'Requests-Tier3'
SA_DATA_TRANSFER_OUT_BYTES = 'SA-DataTransfer-Out-Bytes'
SA_REQUEST_TIER1 = 'SA-Requests-Tier1'
SA_REQUESTS_TIER2_HTTPS = 'SA-Requests-Tier2-HTTPS'
SAE1_AWS_IN_BYTES = 'SAE1-AWS-In-Bytes'
SAE1_AWS_OUT_BYTES = 'SAE1-AWS-Out-Bytes'
SSL_CERT_CUSTOM = 'SSL-Cert-Custom'
SPOT_USAGE_M1_MEDIUM = 'SpotUsage:m1.medium'
SPOT_USAGE_T1_MICRO = 'SpotUsage:t1.micro'
STORAGE_SHARD_HOUR = 'Storage-ShardHour'
T1_MICRO = 't1.micro'
T1_MICRO_RHEL = 't1.micro.rhel'
T1_MICRO_WINDOWS = 't1.micro.windows'
TIMED_STORAGE_BYTE_HRS = 'TimedStorage-ByteHrs'
TIMED_STORAGE_GLACIER_BYTE_HRS = 'TimedStorage-GlacierByteHrs'
TIME_STORAGE_RRS_BYTE_HRS = 'TimedStorage-RRS-ByteHrs'
UGW1_AWS_IN_BYTES = 'UGW1-AWS-In-Bytes'
UGW1_AWS_OUT_BYTES = 'UGW1-AWS-Out-Bytes'
US_DATA_TRANSFER_OUT_BYTES = 'US-DataTransfer-Out-Bytes'
US_REQUESTS_TIER1 = 'US-Requests-Tier1'
US_REQUESTS_TIER2_HTTPS = 'US-Requests-Tier2-HTTPS'
USE1_AWS_IN_BYTES = 'USE1-AWS-In-Bytes'
USE1_AWS_OUT_BYTES = 'USE1-AWS-Out-Bytes'
USW1_AWS_IN_BYTES = 'USW1-AWS-In-Bytes'
USW1_AWS_OUT_BYTES = 'USW1-AWS-Out-Bytes'
USW2_AWS_IN_BYTES = 'USW2-AWS-In-Bytes'
USW2_AWS_OUT_BYTES = 'USW2-AWS-Out-Bytes'
VPN_USAGE_HOURS_IPSEC_1 = 'VPN-Usage-Hours:ipsec.1'
WRITE_CAPACITY_UNIT_HRS = 'WriteCapacityUnit-Hrs'



"""
Ice Pick
~~~~~~~~~~~~~~~~~~~~

Ice Pick is a library that allows easy access to AWS billing data collected by
the Netflix OSS Ice tool.

Basic usage:

>>> from ice_pick.api import APIRequest

>>> api_request = APIRequest('http://example.com/ice/')
>>> api_request.get_data()
{
    'data': {
        'AWS Data Pipeline': [0.25148882999999966],
        'Alexa Web Information Service': [0.11130000000000001],
        'aggregated': [48884.90918908445],
        'cloudfront': [3241.575401989994],
        .
        .
        .
        'vpc': [296.7824844800026]
    },
    'groupBy': 'Product',
    'hours': [588],
    'start': 1391212800000,
    'stats': {
        'AWS Data Pipeline': {
            'average': 0.25148882999999966,
            'max': 0.25148882999999966,
            'total': 0.25148882999999966
        },
        'Alexa Web Information Service': {
            'average': 0.11130000000000001,
            'max': 0.11130000000000001,
            'total': 0.11130000000000001
        },
        'aggregated': {
            'average': 48884.90918908445,
            'max': 48884.90918908445,
            'total': 48884.90918908445
        },
        'cloudfront': {
            'average': 3241.575401989994,
            'max': 3241.575401989994,
            'total': 3241.575401989994
        },
        .
        .
        .
        'vpc': {
            'average': 296.7824844800026,
            'max': 296.7824844800026,
            'total': 296.7824844800026
            }
        },
    'status': 200,
    'time': [1391212800000]
}

Copyright 2014 Demand Media.

    Licensed under the Apache License, Version 2.0 (the "License");
    you may not use this file except in compliance with the License.
    You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

    Unless required by applicable law or agreed to in writing, software
    distributed under the License is distributed on an "AS IS" BASIS,
    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
    See the License for the specific language governing permissions and
    limitations under the License.

"""


__title__ = 'ice_pick'
__version__ = '0.4'
__author__ = 'Demand Media'
__license__ = 'Apache 2.0'
__copyright__ = 'Copyright 2014 Demand Media'



"""
ice_pick.api
~~~~~~~~~~~~

This module implements the Ice Pick API.

Copyright 2014 Demand Media.

    Licensed under the Apache License, Version 2.0 (the "License");
    you may not use this file except in compliance with the License.
    You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

    Unless required by applicable law or agreed to in writing, software
    distributed under the License is distributed on an "AS IS" BASIS,
    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
    See the License for the specific language governing permissions and
    limitations under the License.

"""


import datetime
import json as _json
import urlparse as _urlparse
import requests as _requests
import exceptions as _exceptions
import utils as _utils
from filters import group_by as _group_by, consolidate as _consolidate


class APIFilters(object):
    ACCOUNTS = 'account'
    AGGREGATE = 'aggregate'
    BREAKDOWN = 'breakdown'
    CONSOLIDATE = 'consolidate'
    END = 'end'
    FACTOR_SPS = 'factorsps'
    GROUP_BY = 'groupBy'
    IS_COST = 'isCost'
    OPERATIONS = 'operation'
    PRODUCTS = 'product'
    REGIONS = 'region'
    SHOW_SPS = 'showsps'
    START = 'start'
    USAGE_TYPES = 'usageType'

    # Filter key type map
    TYPES = {
        ACCOUNTS: list,
        AGGREGATE: str,
        BREAKDOWN: bool,
        CONSOLIDATE: str,
        END: datetime.datetime,
        FACTOR_SPS: bool,
        GROUP_BY: str,
        IS_COST: bool,
        OPERATIONS: list,
        PRODUCTS: list,
        REGIONS: list,
        SHOW_SPS: bool,
        START: datetime.datetime,
        USAGE_TYPES: list,
    }

    @classmethod
    def default_filters(cls):
        end_datetime = datetime.datetime.utcnow()
        start_datetime = datetime.datetime(end_datetime.year,
                                           end_datetime.month, 1)
        return {
            cls.AGGREGATE: 'data',
            cls.BREAKDOWN: True,
            cls.CONSOLIDATE: _consolidate.MONTHLY,
            cls.START: _utils.format_datetime(start_datetime),
            cls.END: _utils.format_datetime(end_datetime),
            cls.FACTOR_SPS: False,
            cls.GROUP_BY: _group_by.PRODUCT,
            cls.IS_COST: True,
            cls.SHOW_SPS: False,
        }


class APIRequest(object):
    '''Handles Ice "scraping" as and API.'''
    # Ice monitoring base URL
    ice_use = None
    # Current filters to be apply or already applied
    _filters = None
    # Last data fetch from Ice
    _data = None

    def __init__(self, ice_url, **filters):
        ''':params ice_url: base URL to your Ice instance. It must include\
        "http" or "https://" and end with "/".
        :type ice_url: str or unicode.
        '''
        self.auth = None
        self.ice_url = ice_url
        self._filters = APIFilters.default_filters()
        self._set_filters(**filters)

    def set_http_auth(self, auth):
        """
        Set the value of the optional 'auth' parameter on
        requests.post calls to get data. See
        http://docs.python-requests.org/en/latest/user/authentication/
        for more information.

        :param auth: requests module auth parameter
        :type auth: mixed
        """
        self.auth = auth

    def _set_filters(self, **filters):
        for filter_key, value in filters.iteritems():
            if not filter_key in APIFilters.TYPES:
                self._filters[filter_key] = value
                continue
            self._set_one_filter(filter_key, value)

    def _set_one_filter(self, filter_key, value):
        filter_type = APIFilters.TYPES[filter_key]
        fn = None
        if filter_type == list:
            fn = self._set_filter_list
        elif filter_type == bool:
            fn = self._set_filter_bool
        elif filter_type == datetime.datetime:
            fn = self._set_filter_datetime
        else:
            fn = self._set_filter_str
        fn(filter_key, value)

    def _set_filter_list(self, filter_key, value_list):
        if isinstance(value_list, list):
            self._filters[filter_key] = ','.join(value_list)
        else:
            raise TypeError('Parameter %s must be a list.' % filter_key)

    def _set_filter_bool(self, filter_key, value):
        if isinstance(value, bool):
            self._filters[filter_key] = value
        else:
            raise TypeError('Parameter %s must be a boolean.' % filter_key)

    def _set_filter_datetime(self, filter_key, value):
        if isinstance(value, datetime.datetime):
            self._filters[filter_key] = _utils.format_datetime(value)
        else:
            raise TypeError('Parameter %s must be a datetime.' % filter_key)

    def _set_filter_str(self, filter_key, value):
        if isinstance(value, str):
            self._filters[filter_key] = value
        else:
            raise TypeError('Parameter %s must be a string.' % filter_key)

    def set_accounts(self, account_list):
        '''Parses the account list and adds it to the filters'''
        self._set_one_filter(APIFilters.ACCOUNTS, account_list)

    def set_aggregate(self, aggregate):
        self._set_one_filter(APIFilters.AGGREGATE, aggregate)

    def set_breakdown(self, breakdown):
        self._set_one_filter(APIFilters.BREAKDOWN, breakdown)

    def set_consolidate(self, consolidate):
        self._set_one_filter(APIFilters.CONSOLIDATE, consolidate)

    def set_end(self, end):
        self._set_one_filter(APIFilters.END, end)

    def set_factor_sps(self, factor_sps):
        self._set_one_filter(APIFilters.FACTOR_SPS, factor_sps)

    def set_group_by(self, group_by):
        self._set_one_filter(APIFilters.GROUP_BY, group_by)

    def set_is_cost(self, is_cost):
        self._set_one_filter(APIFilters.IS_COST, is_cost)

    def set_operations(self, operation_list):
        '''Parses the operation list and adds it to the filters'''
        self._set_one_filter(APIFilters.OPERATIONS, operation_list)

    def set_products(self, product_list):
        '''Parses the product list and adds it to the filters'''
        self._set_one_filter(APIFilters.PRODUCTS, product_list)

    def set_regions(self, region_list):
        '''Parses the region list and adds it to the filters'''
        self._set_one_filter(APIFilters.REGIONS, region_list)

    def set_show_sps(self, show_sps):
        self._set_one_filter(APIFilters.SHOW_SPS, show_sps)

    def set_start(self, start):
        self._set_one_filter(APIFilters.START, start)

    def set_usage_types(self, usage_type_list):
        '''Parses the usage type list and adds it to the filters'''
        self._set_one_filter(APIFilters.USAGE_TYPES, usage_type_list)

    def get_filters(self):
        ''' Returns a copy of the current API filters.'''
        return self._filters.copy()

    def get_data(self):
        '''Fetches data from Ice and returns it as a dictonary'''
        request_url = _urlparse.urljoin(self.ice_url, 'dashboard/getData')
        data_filters = _json.dumps(self._filters)
        headers = {
            'content-type': 'application/json;charset=utf-8'
        }
        r = _requests.post(request_url, data=data_filters, headers=headers, auth=self.auth)
        status_code = r.status_code
        if status_code == 200:
            response = r.content
            data = _json.loads(response)
            return data
        raise _exceptions.APIRequestException('POST', request_url,
                                              status_code)



"""
ice_pick.groups
~~~~~~~~~~~~~~~~

This module implements the Ice Pick API for managing Application Groups.
"""

import requests as _requests
import re
import json
import datetime
from collections import defaultdict
import urlparse as _urlparse
from exceptions import APIRequestException
from api import APIRequest
from filters import group_by as _group_by

# Set default logging handler to avoid "No handler found" warnings.
import logging
try: # Python 2.7+
    from logging import NullHandler
except ImportError:
    class NullHandler(logging.Handler):
        def emit(self, record):
            pass


logger = logging.getLogger(__name__)
logger.addHandler(NullHandler())


class Groups(object):
    """
    Class to manage Netflix Ice Application Groups.

    :param ice_host: hostname or IP address of Ice instance
    :type ice_host: string
    :param logger: the logger instance to log to
    :type logger: logging.Logger
    :param dry_run: if true, do not make any *changes* via API, only report on status
      and what would be done
    :type dry_run: boolean
    """

    url_re = re.compile(r'^https?://\S+/$')

    def __init__(self, ice_url, dry_run=False):
        """
        Query and manage Ice Application and Resource Groups.

        :param ice_url: base URL to your Ice instance. It must include\
        "http" or "https://" and end with "/".
        :type ice_url: str or unicode
        :param dry_run: if True, don't actually change anything in Ice,
        just log what would be done (default False).
        :type dry_run: boolean
        """
        self.dry_run = dry_run
        self.ice_url = ice_url
        self.auth = None
        if not self.url_re.match(ice_url):
            raise ValueError("ice_url must match {re}".format(re=self.url_re.pattern))
        if self.dry_run:
            logger.warning("DRY RUN only - will not make any changes")

    def set_http_auth(self, auth):
        """
        Set the value of the optional 'auth' parameter on
        requests.post calls to get data. See
        http://docs.python-requests.org/en/latest/user/authentication/
        for more information.

        :param auth: requests module auth parameter
        :type auth: mixed
        """
        self.auth = auth

    def _ice_get(self, path):
        """
        Perform a GET request against Ice, for the given path/URL.
        Return the decoded data.

        :param path: the path (relative to ice_url) to GET
        :type path: string
        :returns: decoded JSON data dict
        :rtype: dict
        :raises: exceptions.APIRequestException
        """
        url = _urlparse.urljoin(self.ice_url + 'dashboard/', path)
        logger.debug("GETing {u}".format(u=url))
        res = _requests.get(url, auth=self.auth)
        if res.status_code != 200:
            raise APIRequestException('GET', url, res.status_code)
        resp = res.json()
        if resp['status'] != 200:
            raise APIRequestException('GET', url, resp['status'])
        if 'data' in resp:
            return resp['data']
        return {}

    def _ice_post(self, path, params):
        """
        Perform a POST request against Ice, for the given path/URL, with the given params.
        Return the decoded data.

        :param path: the path (relative to ice_url) to POST to
        :type path: string
        :param params: dict of POST data
        :type params: dict
        :returns: decoded JSON response data dict
        :rtype: dict
        :raises: exceptions.APIRequestException
        """
        url = _urlparse.urljoin(self.ice_url + 'dashboard/', path)
        if self.dry_run:
            logger.warning("DRY RUN: Would POST to {u}: {p}".format(u=url, p=params))
            return
        logger.debug("POSTing to {u}: {p}".format(u=url, p=params))
        res = _requests.post(url, data=json.dumps(params), auth=self.auth)
        if res.status_code != 200:
            raise APIRequestException('POST', url, res.status_code)
        resp = res.json()
        if resp['status'] != 200:
            raise APIRequestException('POST', url, resp['status'])
        if 'data' in resp:
            return resp['data']
        return {}

    def get_account_names(self):
        """
        Get a list of all configured AWS account names

        :returns: list of account names
        :rtype: list of strings
        """
        return [x['name'] for x in self._ice_get('getAccounts?')]

    def get_regions_for_account(self, accounts):
        """
        Get a list of AWS region names for the given account.

        :param accounts: list of account names, as returned in ``get_account_names()``
        :type accounts: list of strings
        :returns: list of regions for the account
        :rtype: list of strings
        """
        if not isinstance(accounts, list):
            raise TypeError("accounts parameter must be a list of strings")
        return [r['name'] for r in self._ice_get('getRegions?account={acct}'.format(acct='%2C'.join(accounts)))]

    def get_all_resource_groups(self, accounts, regions, products):
        """
        Get all Resource Group names from Ice.
        This is a wrapper arouns 

        :param accounts: list of account names, as returned in ``get_account_names()``
        :type accounts: list of strings
        :param regions: list of regions to get resource groups for
        :type regions: list of strings
        :param products: list of product names to get resource groups for
        :type products: list of strings
        :returns: list of resource group names
        :rtype: list of strings
        """
        if not isinstance(accounts, list):
            raise TypeError("accounts parameter must be a list of strings")
        url = 'getResourceGroups?account={acct}&product={products}&region={regions}'.format(
            acct='%2C'.join(accounts),
            regions='%2C'.join(regions),
            products='%2C'.join(products)
        )
        return [g['name'] for g in self._ice_get(url)]

    def get_products(self, accounts, regions):
        """
        Get a list of product name strings for the given account and region.

        :param accounts: list of account names, as returned in ``get_account_names()``
        :type accounts: list of strings
        :param regions: regions to query data for
        :type regions: list of strings
        :returns: list of product names
        :rtype: list of strings
        """
        if not isinstance(accounts, list):
            raise TypeError("accounts parameter must be a list of strings")
        path = 'getProducts?account={acct}&region={regions}&showResourceGroups=true'.format(
            acct='%2C'.join(accounts),
            regions='%2C'.join(regions)
        )
        return [r['name'] for r in self._ice_get(path)]

    def get_resource_group_lists(self):
        """
        Return a dict of Ice products (string) to their possible resource groups (list of strings).

        :returns: dict of Ice products (string) to their possible resource groups (list of strings)
        :rtype: dict
        """
        resp = self._ice_get('getResourceGroupLists?')
        result = {}
        for item in resp:
            result[item['product']['name']] = [r['name'] for r in item['data']]
        return result

    def delete_application_group(self, name):
        """
        Delete the specified application group.

        :param name: name of the application group to delete
        :type name: string
        """
        url = 'deleteApplicationGroup?name={name}'.format(name=name)
        if self.dry_run:
            logger.warning("Would GET {u}".format(u=url))
            return
        self._ice_get(url)

    def get_application_group_names(self):
        """
        Return a list of the names of all current Application Groups.

        :rtype: list of strings
        """
        end_dt = datetime.datetime.utcnow()
        start_dt = end_dt - datetime.timedelta(weeks=4)
        req = APIRequest(self.ice_url)
        if self.auth:
            req.set_http_auth(self.auth)
        req.set_aggregate('stats')
        req.set_group_by(_group_by.APPLICATION_GROUP)
        req.set_end(end_dt)
        req.set_start(start_dt)
        result = req.get_data()
        names = [k for k in result['stats']]
        return names

    def get_application_group(self, group_name):
        """
        Get the configuration for the specified Application Group.

        Returns a dict with keys:
        'name' -> string name of group
        'owner' -> string email address of group owner/contact
        'products' -> dict of product name (string) to list of Resource Group names (string) for product

        :param group_name: name of the application group to get
        :type group_name: string
        :rtype: dict
        """
        res = self._ice_get('getApplicationGroup?name={name}'.format(name=group_name))
        group = {}
        group['name'] = res['name']
        group['owner'] = res['owner']
        group['products'] = {}
        for prodname, prodlist in res['data'].iteritems():
            group['products'][prodname] = [x['name'] for x in prodlist]
        return group

    def set_application_group(self, group_name, resource_groups, group_email):
        """
        Create or update an Application Group.

        :param group_name: the name of the group to update
        :type group_name: string
        :param group_email: the owner/contact email for the group
        :type group_email: string
        :param resource_groups: dict of product name to Resource Groups for the product, like the 'products' item in the ``get_application_group()`` return dict
        :type resource_groups: dict of string keys, list of string values
        :returns: Ice POST response data
        :rtype: dict
        """
        params = {"name":group_name, "owner":group_email, "data": resource_groups}
        if self.dry_run:
            logger.warning("DRY_RUN: would POST to /saveApplicationGroup with data: {d}".format(d=params))
            return
        logger.info("POSTing to /saveApplicationGroup with data: {d}".format(d=params))
        res = self._ice_post('saveApplicationGroup', params)
        return res



"""
ice_pick.filters.regions
~~~~~~~~~~~~~~~~~~~~~~~~

This module contains a set of constants that can be used as region
filters. All the constants are based on AWS definitions.

Copyright 2014 Demand Media.

    Licensed under the Apache License, Version 2.0 (the "License");
    you may not use this file except in compliance with the License.
    You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

    Unless required by applicable law or agreed to in writing, software
    distributed under the License is distributed on an "AS IS" BASIS,
    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
    See the License for the specific language governing permissions and
    limitations under the License.
"""


AP_NORTHEAST_1 = 'ap-northeast-1'  # Asia Pacific (Tokyo) Region
AP_SOUTHEAST_1 = 'ap-southeast-1'  # Asia Pacific (Singapore) Region
AP_SOUTHEAST_2 = 'ap-southeast-2'  # Asia Pacific (Sydney) Region
EU_WEST_1 = 'eu-west-1'  # EU (Ireland) Region
SA_EAST_1 = 'sa-east-1'  # South America (Sao Paulo) Region
US_EAST_1 = 'us-east-1'  # US East (Northern Virginia) Region
US_WEST_1 = 'us-west-1'  # US West (Northern California) Region
US_WEST_2 = 'us-west-2'  # US West (Oregon) Region



import unittest
from mock import patch, call, Mock, MagicMock
import datetime
from ice_pick import utils
from ice_pick.exceptions import APIRequestException
from ice_pick import api
from ice_pick.api import APIRequest, APIFilters
from ice_pick.groups import Groups
import logging
from contextlib import nested
from ice_pick.filters import group_by as _group_by
from freezegun import freeze_time


class TestUtils(unittest.TestCase):
    def test_format_datetime(self):
        '''Tests that the datetime formatter utils function returns the right
        format.
        '''
        utcnow = datetime.datetime.utcnow()
        datetime_format = '%Y-%m-%d %I%p'
        self.assertEquals(utcnow.strftime(datetime_format),
                          utils.format_datetime(utcnow))


class TestAPIFilters(unittest.TestCase):
    def test_filter_keys(self):
        '''Tests the name of the filter keys.'''
        self.assertEquals(APIFilters.ACCOUNTS, 'account')
        self.assertEquals(APIFilters.AGGREGATE, 'aggregate')
        self.assertEquals(APIFilters.BREAKDOWN, 'breakdown')
        self.assertEquals(APIFilters.CONSOLIDATE, 'consolidate')
        self.assertEquals(APIFilters.END, 'end')
        self.assertEquals(APIFilters.FACTOR_SPS, 'factorsps')
        self.assertEquals(APIFilters.GROUP_BY, 'groupBy')
        self.assertEquals(APIFilters.IS_COST, 'isCost')
        self.assertEquals(APIFilters.OPERATIONS, 'operation')
        self.assertEquals(APIFilters.PRODUCTS, 'product')
        self.assertEquals(APIFilters.REGIONS, 'region')
        self.assertEquals(APIFilters.SHOW_SPS, 'showsps')
        self.assertEquals(APIFilters.START, 'start')
        self.assertEquals(APIFilters.USAGE_TYPES, 'usageType')

    def test_types(self):
        '''Tests the types of the filters.'''
        types = APIFilters.TYPES
        self.assertEquals(types[APIFilters.ACCOUNTS], list)
        self.assertEquals(types[APIFilters.AGGREGATE], str)
        self.assertEquals(types[APIFilters.BREAKDOWN], bool)
        self.assertEquals(types[APIFilters.CONSOLIDATE], str)
        self.assertEquals(types[APIFilters.END], datetime.datetime)
        self.assertEquals(types[APIFilters.FACTOR_SPS], bool)
        self.assertEquals(types[APIFilters.GROUP_BY], str)
        self.assertEquals(types[APIFilters.IS_COST], bool)
        self.assertEquals(types[APIFilters.OPERATIONS], list)
        self.assertEquals(types[APIFilters.PRODUCTS], list)
        self.assertEquals(types[APIFilters.REGIONS], list)
        self.assertEquals(types[APIFilters.SHOW_SPS], bool)
        self.assertEquals(types[APIFilters.START], datetime.datetime)
        self.assertEquals(types[APIFilters.USAGE_TYPES], list)

    def test_default_filters(self):
        '''Tests the default filters key/value.'''
        utcnow = datetime.datetime.utcnow()
        start = datetime.datetime(utcnow.year, utcnow.month, 1)
        default_key_value = {
            APIFilters.IS_COST: True,
            APIFilters.AGGREGATE: 'data',
            APIFilters.GROUP_BY: 'Product',
            APIFilters.CONSOLIDATE: 'monthly',
            APIFilters.START: utils.format_datetime(start),
            APIFilters.END: utils.format_datetime(utcnow),
            APIFilters.BREAKDOWN: True,
            APIFilters.SHOW_SPS: False,
            APIFilters.FACTOR_SPS: False,
        }
        assert APIFilters.default_filters() == default_key_value


class TestAPIRequest(unittest.TestCase):
    api_request = None
    dummy_ice_url = None
    dummy_list = None
    dummy_string = None
    dummy_datetime = None

    def setUp(self):
        self.dummy_ice_url = 'http://foo.com'
        self.api_request = APIRequest(self.dummy_ice_url)
        self.dummy_list = ['012345678900', '009876543210']
        self.dummy_str = 'foo'
        self.dummy_datetime = datetime.datetime.utcnow()

    def _join_list(self, value_list):
        return ','.join(value_list)

    def test_api_request_filter(self):
        is_cost = False
        dummy_filter = 'foo_bar'
        dummy_value = ['foo', 'bar']
        filters = {
            APIFilters.ACCOUNTS: self.dummy_list,
            APIFilters.IS_COST: is_cost,
            dummy_filter: dummy_value,
        }
        api_request = APIRequest(self.dummy_ice_url, **filters)
        request_filters = api_request.get_filters()
        self.assertEquals(request_filters[APIFilters.ACCOUNTS],
                          self._join_list(self.dummy_list))
        self.assertEquals(request_filters[APIFilters.IS_COST], is_cost)
        self.assertEquals(request_filters[dummy_filter], dummy_value)

    def test_set_http_auth(self):
        api_request = APIRequest(self.dummy_ice_url)
        self.assertEquals(api_request.auth, None)
        api_request.set_http_auth(('foo', 'bar'))
        self.assertEquals(api_request.auth, ('foo', 'bar'))

    def _test_set_list(self, fn, filter_name):
        # Type error exception test
        self.assertRaises(TypeError,
                          fn,
                          self.dummy_str)
        # Filter value test
        fn(self.dummy_list)
        self.assertEquals(self.api_request.get_filters()[filter_name],
                          self._join_list(self.dummy_list))

    def _test_set_str(self, fn, filter_name):
        # Type error exception test
        self.assertRaises(TypeError, fn, [])
        # Filter value test
        fn(self.dummy_str)
        self.assertEquals(self.api_request.get_filters()[filter_name],
                          self.dummy_str)

    def _test_set_bool(self, fn, filter_name, value):
        # Type error exception test
        self.assertRaises(TypeError, fn, self.dummy_str)
        # Filter value test
        fn(value)
        self.assertEquals(self.api_request.get_filters()[filter_name], value)

    def _test_set_datetime(self, fn, filter_name):
        # Type error exception test
        self.assertRaises(TypeError, fn, self.dummy_str)
        # Filter value test
        fn(self.dummy_datetime)
        self.assertEquals(self.api_request.get_filters()[filter_name],
                          utils.format_datetime(self.dummy_datetime))

    def test_set_accounts(self):
        '''Tests set_accounts function exception and filter results'''
        self._test_set_list(self.api_request.set_accounts, APIFilters.ACCOUNTS)

    def test_set_aggregate(self):
        '''Tests set_aggregate function exception and filter results'''
        self._test_set_str(self.api_request.set_aggregate,
                           APIFilters.AGGREGATE)

    def test_set_breakdown(self):
        '''Tests set_breakdown function exception and filter results'''
        self._test_set_bool(self.api_request.set_breakdown,
                            APIFilters.BREAKDOWN, False)

    def test_set_consolidate(self):
        '''Tests set_consolidate function exception and filter results'''
        self._test_set_str(self.api_request.set_consolidate,
                           APIFilters.CONSOLIDATE)

    def test_set_end(self):
        '''Tests set_end function exception and filter results'''
        self._test_set_datetime(self.api_request.set_end, APIFilters.END)

    def test_set_factor_sps(self):
        '''Tests set_factor_sps function exception and filter results'''
        self._test_set_bool(self.api_request.set_factor_sps,
                            APIFilters.FACTOR_SPS, True)

    def test_set_group_by(self):
        '''Tests set_group_by function exception and filter results'''
        self._test_set_str(self.api_request.set_group_by,
                           APIFilters.GROUP_BY)

    def test_set_is_cost(self):
        '''Tests set_is_cost function exception and filter results'''
        self._test_set_bool(self.api_request.set_is_cost,
                            APIFilters.IS_COST, False)

    def test_set_operations(self):
        '''Tests set_operations function exception and filter results'''
        self._test_set_list(self.api_request.set_operations,
                            APIFilters.OPERATIONS)

    def test_set_products(self):
        '''Tests set_products function exception and filter results'''
        self._test_set_list(self.api_request.set_products,
                            APIFilters.PRODUCTS)

    def test_set_regions(self):
        '''Tests set_regions function exception and filter results'''
        self._test_set_list(self.api_request.set_regions,
                            APIFilters.REGIONS)

    def test_set_show_sps(self):
        '''Tests set_show_sps function exception and filter results'''
        self._test_set_bool(self.api_request.set_show_sps,
                            APIFilters.SHOW_SPS, True)

    def test_set_start(self):
        '''Tests set_start function exception and filter results'''
        self._test_set_datetime(self.api_request.set_start, APIFilters.START)

    def test_set_usage_types(self):
        '''Tests set_usage_types function exception and filter results'''
        self._test_set_list(self.api_request.set_usage_types,
                            APIFilters.USAGE_TYPES)

    def test_get_data(self):
        '''Tests the get_data function by mocking the requests.post'''
        class MockResponse():
            def __init__(self, status_code):
                self.status_code = status_code

            @property
            def content(self):
                return '''{
                    "data": {},
                    "groupBy": {},
                    "interval": 3600000,
                    "start": 1380585600000,
                    "stats": {},
                    "status": 200
                }'''

        with patch.object(api, '_requests') as mock_request:
            # Testing that if the post request comes back with a status_code
            # different than 200, the function get_data will raise an
            # APIRequestException.
            mock_request.post.return_value = MockResponse(500)
            self.assertRaises(APIRequestException, self.api_request.get_data)

            # Testing that if the post request status_code is 200, the function
            # get_data will return a dictionary.
            mock_request.post.return_value = MockResponse(200)
            self.assertTrue(isinstance(self.api_request.get_data(), dict))

@patch('ice_pick.groups.logger', spec_set=logging.Logger)
@patch('ice_pick.groups._requests')
@patch('ice_pick.groups.Groups._ice_get')
@patch('ice_pick.groups.Groups._ice_post')
class TestGroups(unittest.TestCase):

    def setUp(self):
        self.dummy_ice_url = 'http://foo.com/'

    def test_init(self, mock_post, mock_get, mock_requests, mock_logger):
        g = Groups(self.dummy_ice_url)
        self.assertEquals(g.ice_url, self.dummy_ice_url)
        self.assertEquals(g.dry_run, False)
        self.assertEquals(g.auth, None)
        self.assertEquals(mock_logger.mock_calls, [])
        self.assertEquals(mock_requests.mock_calls, [])
        self.assertEquals(mock_get.mock_calls, [])
        self.assertEquals(mock_post.mock_calls, [])

    def test_init_bad_url(self, mock_post, mock_get, mock_requests, mock_logger):
        self.assertRaises(ValueError, Groups, 'foobar')
        self.assertEquals(mock_logger.mock_calls, [])
        self.assertEquals(mock_requests.mock_calls, [])
        self.assertEquals(mock_get.mock_calls, [])
        self.assertEquals(mock_post.mock_calls, [])

    def test_init_dry_run(self, mock_post, mock_get, mock_requests, mock_logger):
        g = Groups(self.dummy_ice_url, dry_run=True)
        self.assertEquals(g.dry_run, True)
        self.assertEquals(mock_logger.mock_calls,
                          [call.warning('DRY RUN only - will not make any changes')]
        )
        self.assertEquals(mock_requests.mock_calls, [])
        self.assertEquals(mock_get.mock_calls, [])
        self.assertEquals(mock_post.mock_calls, [])

    def test_set_auth(self, mock_post, mock_get, mock_requests, mock_logger):
        m = Mock()
        g = Groups(self.dummy_ice_url)
        g.set_http_auth(m)
        self.assertEquals(g.ice_url, self.dummy_ice_url)
        self.assertEquals(g.dry_run, False)
        self.assertEquals(g.auth, m)
        self.assertEquals(mock_logger.mock_calls, [])
        self.assertEquals(mock_requests.mock_calls, [])
        self.assertEquals(mock_get.mock_calls, [])
        self.assertEquals(mock_post.mock_calls, [])

    def test_get_account_names(self, mock_post, mock_get, mock_requests, mock_logger):
        mock_get.return_value = [
            {'name': '1234'},
            {'name': '5678'}
        ]
        g = Groups(self.dummy_ice_url, dry_run=False)
        res = g.get_account_names()
        self.assertEquals(mock_logger.mock_calls,
                          []
        )
        self.assertEquals(mock_requests.mock_calls, [])
        self.assertEquals(mock_get.mock_calls, [call('getAccounts?')])
        self.assertEquals(mock_post.mock_calls, [])
        self.assertEquals(res, ['1234', '5678'])

    def test_get_regions_for_account(self, mock_post, mock_get, mock_requests, mock_logger):
        mock_get.return_value = [
            {"name":"ap-northeast-1"},
            {"name":"us-east-1"},
            {"name":"us-west-2"}
        ]
        g = Groups(self.dummy_ice_url, dry_run=False)
        res = g.get_regions_for_account(['123456', '7890'])
        self.assertEquals(mock_logger.mock_calls, [])
        self.assertEquals(mock_requests.mock_calls, [])
        self.assertEquals(mock_get.mock_calls, [call('getRegions?account=123456%2C7890')])
        self.assertEquals(mock_post.mock_calls, [])
        self.assertEquals(res, ["ap-northeast-1", "us-east-1", "us-west-2"])

    def test_get_all_resource_groups(self, mock_post, mock_get, mock_requests, mock_logger):
        mock_get.return_value = [
            {"name":"tag1_tagA"},
            {"name":"tag2_tagA"},
            {"name":"tag1_tagB"},
            {"name":"tag2_tagB"},
        ]
        url = 'getResourceGroups?account=23456%2C7890&product=ec2%2Cs3%2Crds&region=us-east-1%2Cus-west-2'
        g = Groups(self.dummy_ice_url, dry_run=False)
        res = g.get_all_resource_groups(['23456', '7890'], ['us-east-1', 'us-west-2'], ['ec2', 's3', 'rds'])
        self.assertEquals(mock_logger.mock_calls, [])
        self.assertEquals(mock_requests.mock_calls, [])
        self.assertEquals(mock_get.mock_calls, [call(url)])
        self.assertEquals(mock_post.mock_calls, [])
        self.assertEquals(res, ["tag1_tagA", "tag2_tagA", "tag1_tagB", "tag2_tagB"])

    def test_get_products(self, mock_post, mock_get, mock_requests, mock_logger):
        mock_get.return_value = [
            {"name":"cloudfront"},
            {"name":"cloudwatch"},
            {"name":"ebs"},
        ]
        url = 'getProducts?account=1234%2C23456&region=us-east-1%2Cus-west-2&showResourceGroups=true'
        g = Groups(self.dummy_ice_url, dry_run=False)
        res = g.get_products(['1234', '23456'], ['us-east-1', 'us-west-2'])
        self.assertEquals(mock_logger.mock_calls, [])
        self.assertEquals(mock_requests.mock_calls, [])
        self.assertEquals(mock_get.mock_calls, [call(url)])
        self.assertEquals(mock_post.mock_calls, [])
        self.assertEquals(res, ["cloudfront", "cloudwatch", "ebs"])

    def test_get_resource_group_lists(self, mock_post, mock_get, mock_requests, mock_logger):
        mock_get.return_value = [
            {
                "product": { "name": "vpc" },
                "data": []
            },
            {
                "product": { "name":"rds" },
                "data":[
                    {"name":"tag1_tagA"},
                    {"name":"tag2_tagA"},
                    {"name":"tag1_tagB"},
                    {"name":"tag2_tagB"},
                ]
            },
            {
                "product": { "name":"ebs" },
                "data":[
                    {"name":"tag1_tagA"},
                    {"name":"tag2_tagA"},
                    {"name":"tag1_tagB"},
                    {"name":"tag2_tagB"},
                ]
            },
        ]
        url = 'getResourceGroupLists?'
        expected = {
            "vpc": [],
            "rds": [
                "tag1_tagA",
                "tag2_tagA",
                "tag1_tagB",
                "tag2_tagB",
            ],
            "ebs": [
                "tag1_tagA",
                "tag2_tagA",
                "tag1_tagB",
                "tag2_tagB",
            ],
        }

        g = Groups(self.dummy_ice_url, dry_run=False)
        res = g.get_resource_group_lists()
        self.assertEquals(mock_logger.mock_calls,
                          []
        )
        self.assertEquals(mock_requests.mock_calls, [])
        self.assertEquals(mock_get.mock_calls, [call(url)])
        self.assertEquals(mock_post.mock_calls, [])
        self.assertEquals(res, expected)

    def test_get_application_group(self, mock_post, mock_get, mock_requests, mock_logger):
        mock_get.return_value = {
            "name": "AppGroup",
            "owner": "nobody@example.com",
            "data": {
                "cloudwatch": [
                    {"name":"tag1_tagA"},
                    {"name":"tag2_tagA"},
                    {"name":"tag1_tagB"},
                    {"name":"tag2_tagB"},
                ],
                "ec2": [
                    {"name":"tag1_tagB"},
                    {"name":"tag2_tagB"},
                ],
                "ebs": [
                    {"name":"tag1_tagA"},
                    {"name":"tag2_tagA"},
                ],
            },
        }
        url = 'getApplicationGroup?name=foobar'
        g = Groups(self.dummy_ice_url, dry_run=False)
        res = g.get_application_group('foobar')
        self.assertEquals(mock_logger.mock_calls,
                          []
        )
        self.assertEquals(mock_requests.mock_calls, [])
        self.assertEquals(mock_get.mock_calls, [call(url)])
        self.assertEquals(mock_post.mock_calls, [])
        self.assertEquals(res, {
            'name': 'AppGroup',
            'owner': 'nobody@example.com',
            'products': {
                "cloudwatch": ["tag1_tagA", "tag2_tagA", "tag1_tagB", "tag2_tagB"],
                "ec2": ["tag1_tagB", "tag2_tagB"],
                "ebs": [ "tag1_tagA", "tag2_tagA"],
            }
        })

    def test_set_application_group(self, mock_post, mock_get, mock_requests, mock_logger):
        products = {
            "cloudwatch": ["tag1_tagA", "tag2_tagA", "tag1_tagB", "tag2_tagB"],
            "ec2": ["tag1_tagB", "tag2_tagB"],
            "ebs": [ "tag1_tagA", "tag2_tagA"],
        }
        params = {'owner': 'foo@example.com', 'name': 'MyName', 'data': products}

        g = Groups(self.dummy_ice_url, dry_run=False)
        g.set_application_group('MyName', products, 'foo@example.com')
        self.assertEquals(mock_logger.mock_calls, [
            call.info("POSTing to /saveApplicationGroup with data: {'owner': "
                      "'foo@example.com', 'data': {'ec2': ['tag1_tagB', 'tag2_tagB'],"
                      " 'cloudwatch': ['tag1_tagA', 'tag2_tagA', 'tag1_tagB', "
                      "'tag2_tagB'], 'ebs': ['tag1_tagA', 'tag2_tagA']},"
                      " 'name': 'MyName'}")
        ])
        self.assertEquals(mock_requests.mock_calls, [])
        self.assertEquals(mock_get.mock_calls, [])
        self.assertEquals(mock_post.mock_calls, [
            call('saveApplicationGroup', params)
        ])

    def test_set_application_group_dry_run(self, mock_post, mock_get, mock_requests, mock_logger):
        products = {
            "cloudwatch": ["tag1_tagA", "tag2_tagA", "tag1_tagB", "tag2_tagB"],
            "ec2": ["tag1_tagB", "tag2_tagB"],
            "ebs": [ "tag1_tagA", "tag2_tagA"],
        }
        params = {'owner': 'foo@example.com', 'name': 'MyName', 'data': products}

        g = Groups(self.dummy_ice_url, dry_run=True)
        mock_logger.reset_mock()
        g.set_application_group('MyName', products, 'foo@example.com')
        self.assertEquals(mock_logger.mock_calls, [
            call.warning("DRY_RUN: would POST to /saveApplicationGroup with data: {'owner': "
                      "'foo@example.com', 'data': {'ec2': ['tag1_tagB', 'tag2_tagB'],"
                      " 'cloudwatch': ['tag1_tagA', 'tag2_tagA', 'tag1_tagB', "
                      "'tag2_tagB'], 'ebs': ['tag1_tagA', 'tag2_tagA']},"
                      " 'name': 'MyName'}")
        ])
        self.assertEquals(mock_requests.mock_calls, [])
        self.assertEquals(mock_get.mock_calls, [])
        self.assertEquals(mock_post.mock_calls, [])

    def test_delete_application_group_dry_run(self, mock_post, mock_get, mock_requests, mock_logger):
        groups = Groups(self.dummy_ice_url, dry_run=True)
        mock_logger.reset_mock()
        groups.delete_application_group('foo')
        self.assertEquals(mock_logger.mock_calls,
                          [call.warning('Would GET deleteApplicationGroup?name=foo')]
        )
        self.assertEquals(mock_get.mock_calls, [])
        self.assertEquals(mock_post.mock_calls, [])

    def test_delete_application_group(self, mock_post, mock_get, mock_requests, mock_logger):
        groups = Groups(self.dummy_ice_url, dry_run=False)
        mock_logger.reset_mock()
        groups.delete_application_group('foo')
        self.assertEquals(mock_logger.mock_calls, [])
        self.assertEquals(mock_get.mock_calls, [call('deleteApplicationGroup?name=foo')])
        self.assertEquals(mock_post.mock_calls, [])


@patch('ice_pick.groups.logger', spec_set=logging.Logger)
@patch('ice_pick.groups._requests')
class TestGroupsRequests(unittest.TestCase):

    def test_ice_get(self, mock_requests, mock_logger):
        url = 'http://foo.com/dashboard/foobar'
        mock_result = Mock(status_code=200)
        mock_result.json.return_value = {"status": 200, "data": ["foo","bar"] }
        mock_requests.get.return_value = mock_result

        g = Groups('http://foo.com/', dry_run=False)
        mock_logger.reset_mock()
        res = g._ice_get('foobar')
        self.assertEquals(mock_logger.mock_calls,
                          [call.debug('GETing http://foo.com/dashboard/foobar')]
        )
        self.assertEquals(mock_requests.mock_calls, [
            call.get('http://foo.com/dashboard/foobar', auth=None),
            call.get().json()
        ])
        self.assertEquals(res, ['foo', 'bar'])

    def test_ice_get_auth(self, mock_requests, mock_logger):
        url = 'http://foo.com/dashboard/foobar'
        mock_result = Mock(status_code=200)
        mock_result.json.return_value = {"status": 200, "data": ["foo","bar"] }
        mock_requests.get.return_value = mock_result

        g = Groups('http://foo.com/', dry_run=False)
        g.set_http_auth(('myuser', 'mypass'))
        mock_logger.reset_mock()
        res = g._ice_get('foobar')
        self.assertEquals(mock_logger.mock_calls,
                          [call.debug('GETing http://foo.com/dashboard/foobar')]
        )
        self.assertEquals(mock_requests.mock_calls, [
            call.get('http://foo.com/dashboard/foobar', auth=('myuser', 'mypass')),
            call.get().json()
        ])
        self.assertEquals(res, ['foo', 'bar'])

    def test_ice_get_request_error(self, mock_requests, mock_logger):
        url = 'http://foo.com/dashboard/foobar'
        mock_result = Mock(status_code=502)
        mock_requests.get.return_value = mock_result

        g = Groups('http://foo.com/', dry_run=False)
        mock_logger.reset_mock()
        self.assertRaises(APIRequestException,
                          g._ice_get,
                          'foobar')
        self.assertEquals(mock_logger.mock_calls, [call.debug('GETing http://foo.com/dashboard/foobar')])
        self.assertEquals(mock_requests.mock_calls, [call.get('http://foo.com/dashboard/foobar', auth=None)])

    def test_ice_get_response_error(self, mock_requests, mock_logger):
        url = 'http://foo.com/dashboard/foobar'
        mock_result = Mock(status_code=200)
        mock_result.json.return_value = {"status": 503, "data": ["foo","bar"] }
        mock_requests.get.return_value = mock_result

        g = Groups('http://foo.com/', dry_run=False)
        mock_logger.reset_mock()
        self.assertRaises(APIRequestException,
                          g._ice_get,
                          'foobar')
        self.assertEquals(mock_logger.mock_calls, [call.debug('GETing http://foo.com/dashboard/foobar')])
        self.assertEquals(mock_requests.mock_calls, [
            call.get('http://foo.com/dashboard/foobar', auth=None),
            call.get().json()
        ])

    def test_ice_get_no_data(self, mock_requests, mock_logger):
        url = 'http://foo.com/dashboard/foobar'
        mock_result = Mock(status_code=200)
        mock_result.json.return_value = {"status": 200}
        mock_requests.get.return_value = mock_result

        g = Groups('http://foo.com/', dry_run=False)
        mock_logger.reset_mock()
        res = g._ice_get('foobar')
        self.assertEquals(mock_logger.mock_calls,
                          [call.debug('GETing http://foo.com/dashboard/foobar')]
        )
        self.assertEquals(mock_requests.mock_calls, [
            call.get('http://foo.com/dashboard/foobar', auth=None),
            call.get().json()
        ])
        self.assertEquals(res, {})

    def test_ice_post(self, mock_requests, mock_logger):
        url = 'http://foo.com/dashboard/foobar'
        mock_result = Mock(status_code=200)
        mock_result.json.return_value = {"status": 200}
        mock_requests.post.return_value = mock_result

        g = Groups('http://foo.com/', dry_run=False)
        mock_logger.reset_mock()
        res = g._ice_post('foobar', {'baz': 'blam'})
        self.assertEquals(mock_logger.mock_calls,
                          [call.debug("POSTing to http://foo.com/dashboard/foobar: {'baz': 'blam'}")]
        )
        self.assertEquals(mock_requests.mock_calls, [
            call.post('http://foo.com/dashboard/foobar', data='{"baz": "blam"}', auth=None),
            call.post().json()
        ])
        self.assertEquals(res, {})

    def test_ice_post_auth(self, mock_requests, mock_logger):
        url = 'http://foo.com/dashboard/foobar'
        mock_result = Mock(status_code=200)
        mock_result.json.return_value = {"status": 200}
        mock_requests.post.return_value = mock_result

        g = Groups('http://foo.com/', dry_run=False)
        g.set_http_auth(('myuser', 'mypass'))
        mock_logger.reset_mock()
        res = g._ice_post('foobar', {'baz': 'blam'})
        self.assertEquals(mock_logger.mock_calls,
                          [call.debug("POSTing to http://foo.com/dashboard/foobar: {'baz': 'blam'}")]
        )
        self.assertEquals(mock_requests.mock_calls, [
            call.post('http://foo.com/dashboard/foobar', data='{"baz": "blam"}', auth=('myuser', 'mypass')),
            call.post().json()
        ])
        self.assertEquals(res, {})

    def test_ice_post_dry_run(self, mock_requests, mock_logger):
        url = 'http://foo.com/dashboard/foobar'

        g = Groups('http://foo.com/', dry_run=True)
        mock_logger.reset_mock()
        res = g._ice_post('foobar', {'baz': 'blam'})
        self.assertEquals(mock_logger.mock_calls,
                          [call.warning("DRY RUN: Would POST to http://foo.com/dashboard/foobar: {'baz': 'blam'}")]
        )
        self.assertEquals(mock_requests.mock_calls, [])

    def test_ice_post_request_error(self, mock_requests, mock_logger):
        url = 'http://foo.com/dashboard/foobar'
        mock_result = Mock(status_code=404)
        mock_requests.post.return_value = mock_result

        g = Groups('http://foo.com/', dry_run=False)
        mock_logger.reset_mock()
        self.assertRaises(APIRequestException,
                          g._ice_post,
                          'foobar',
                          {'baz': 'blam'}
        )
        self.assertEquals(mock_logger.mock_calls,
                          [call.debug("POSTing to http://foo.com/dashboard/foobar: {'baz': 'blam'}")]
        )
        self.assertEquals(mock_requests.mock_calls, [
            call.post('http://foo.com/dashboard/foobar', data='{"baz": "blam"}', auth=None)
        ])

    def test_ice_post_response_error(self, mock_requests, mock_logger):
        url = 'http://foo.com/dashboard/foobar'
        mock_result = Mock(status_code=200)
        mock_result.json.return_value = {"status": 403}
        mock_requests.post.return_value = mock_result

        g = Groups('http://foo.com/', dry_run=False)
        mock_logger.reset_mock()
        self.assertRaises(APIRequestException,
                          g._ice_post,
                          'foobar',
                          {'baz': 'blam'}
        )
        self.assertEquals(mock_logger.mock_calls,
                          [call.debug("POSTing to http://foo.com/dashboard/foobar: {'baz': 'blam'}")]
        )
        self.assertEquals(mock_requests.mock_calls, [
            call.post('http://foo.com/dashboard/foobar', data='{"baz": "blam"}', auth=None),
            call.post().json()
        ])

    def test_ice_post_data(self, mock_requests, mock_logger):
        url = 'http://foo.com/dashboard/foobar'
        mock_result = Mock(status_code=200)
        mock_result.json.return_value = {"status": 200, "data": {'foo': 'bar'}}
        mock_requests.post.return_value = mock_result

        g = Groups('http://foo.com/', dry_run=False)
        mock_logger.reset_mock()
        res = g._ice_post('foobar', {'baz': 'blam'})
        self.assertEquals(mock_logger.mock_calls,
                          [call.debug("POSTing to http://foo.com/dashboard/foobar: {'baz': 'blam'}")]
        )
        self.assertEquals(mock_requests.mock_calls, [
            call.post('http://foo.com/dashboard/foobar', data='{"baz": "blam"}', auth=None),
            call.post().json()
        ])
        self.assertEquals(res, {'foo': 'bar'})

    @freeze_time('2015-04-10 05:00:00')
    def test_get_application_group_names(self, mock_requests, mock_logger):
        mock_api = Mock()
        mock_api.get_data.return_value = {
            u'status': 200,
            u'stats': {
                u'foo': {
                    u'max': 21.597462450416614,
                    u'average': 21.597462450416614,
                    u'total': 21.597462450416614
                },
                u'bar': {
                    u'max': 1.6682055999999998,
                    u'average': 1.6682055999999998,
                    u'total': 1.6682055999999998
                },
                u'baz': {
                    u'max': 26.88331985366129,
                    u'average': 26.88331985366129,
                    u'total': 26.88331985366129
                },
            },
            u'hours': [151],
            u'start': 1430438400000,
            u'time': [1430438400000],
            u'data': {
                u'foo': [21.597462450416614],
                u'bar': [1.6682055999999998],
                u'baz': [26.88331985366129],
            },
            u'groupBy': u'ApplicationGroup'
        }
        
        with patch('ice_pick.groups.APIRequest', spec_set=APIRequest) as mock_api_constr:
            mock_api_constr.return_value = mock_api
            g = Groups('http://foo.com/', dry_run=False)
            res = g.get_application_group_names()
        self.assertEqual(mock_api_constr.mock_calls[0], call('http://foo.com/'))
        self.assertEqual(mock_api.mock_calls, [
            call.set_aggregate('stats'),
            call.set_group_by(_group_by.APPLICATION_GROUP),
            call.set_end(datetime.datetime(2015, 4, 10, 5, 0)),
            call.set_start(datetime.datetime(2015, 3, 13, 5, 0)),
            call.get_data(),
        ])
        self.assertEqual(sorted(res), [u'bar', u'baz', u'foo'])
        self.assertEqual(mock_logger.mock_calls, [])

    @freeze_time('2015-04-10 05:00:00')
    def test_get_application_group_names_auth(self, mock_requests, mock_logger):
        mock_api = Mock()
        mock_api.get_data.return_value = {
            u'status': 200,
            u'stats': {
                u'foo': {
                    u'max': 21.597462450416614,
                    u'average': 21.597462450416614,
                    u'total': 21.597462450416614
                },
                u'bar': {
                    u'max': 1.6682055999999998,
                    u'average': 1.6682055999999998,
                    u'total': 1.6682055999999998
                },
                u'baz': {
                    u'max': 26.88331985366129,
                    u'average': 26.88331985366129,
                    u'total': 26.88331985366129
                },
            },
            u'hours': [151],
            u'start': 1430438400000,
            u'time': [1430438400000],
            u'data': {
                u'foo': [21.597462450416614],
                u'bar': [1.6682055999999998],
                u'baz': [26.88331985366129],
            },
            u'groupBy': u'ApplicationGroup'
        }
        with patch('ice_pick.groups.APIRequest', spec_set=APIRequest) as mock_api_constr:
            mock_api_constr.return_value = mock_api
            g = Groups('http://foo.com/', dry_run=False)
            g.set_http_auth(('user', 'pass'))
            res = g.get_application_group_names()
        self.assertEqual(mock_api_constr.mock_calls[0], call('http://foo.com/'))
        self.assertEqual(mock_api.mock_calls, [
            call.set_http_auth(('user', 'pass')),
            call.set_aggregate('stats'),
            call.set_group_by(_group_by.APPLICATION_GROUP),
            call.set_end(datetime.datetime(2015, 4, 10, 5, 0)),
            call.set_start(datetime.datetime(2015, 3, 13, 5, 0)),
            call.get_data(),
        ])
        self.assertEqual(sorted(res), [u'bar', u'baz', u'foo'])
        self.assertEqual(mock_logger.mock_calls, [])

if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python

import ice_pick

try:
    from setuptools import setup
except ImportError:
    from distutils.core import setup

packages = [
    'ice_pick',
    'ice_pick.filters',
]

test_requires = []
with open('requirements_test.txt') as f:
    test_requires = f.read().split('\n')

requires = []
with open('requirements.txt') as f:
    requires = f.read().split('\n')

setup(name=ice_pick.__title__,
      version=ice_pick.__version__,
      description='Python data access interface for Netflix OSS Ice Tool',
      author=ice_pick.__author__,
      author_email='opensource@demandmedia.com',
      url='https://github.com/demandmedia/ice_pick',
      license='Apache 2.0',
      packages=packages,
      package_dir={'ice_pick': 'ice_pick'},
      install_requires=requires,
      tests_require=test_requires,
      setup_requires=['nose==1.3.0'],
      classifiers=('Development Status :: 5 - Production/Stable',
                   'Intended Audience :: Developers',
                   'Natural Language :: English',
                   'License :: OSI Approved :: Apache Software License',
                   'Programming Language :: Python',
                   'Programming Language :: Python :: 2.6',
                   'Programming Language :: Python :: 2.7',
      )
)
